'' =================================================================================================
''
''   File....... jm_165_fast.spin
''   Purpose.... 74x165 shift register reader
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2014 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 19 JAN 2014
''
'' =================================================================================================


con { fixed io pins }

  RX1     = 31                                                  ' programming / terminal
  TX1     = 30
  
  SDA     = 29                                                  ' eeprom / i2c
  SCL     = 28


con

  #0, LSBFIRST, MSBFIRST


var

  long  cog

  long  cmd
  long  bits


pub start(dpin, cpin, lpin)

'' Start 74x165 reader cog
'' -- dpin is data pin (9) from x165
'' -- cpin is clock pin (2) to x165
'' -- lpin is shift/load pin (1)  to x164

  stop                                                          ' stop if previously loaded
  
  cmd.byte[0] := dpin                                           ' pack pins
  cmd.byte[1] := cpin
  cmd.byte[2] := lpin

  cog := cognew(@entry, @cmd) + 1                               ' start the cog

  return cog
  

pub stop

'' Stop encoder cog if running

  if (cog)
    cogstop(cog - 1)
    cog := false


pub read(mode, count)

'' Input 1 to 4 bytes from 74x165 chain
'' -- mode is 0 for LSBFIRST, 1 for MSBFIRST (standard)
'' -- count is number of bytes (1 to 4)

  count <<= 3                                                   ' convert bytes to bits (x8)

  cmd := count
  repeat
  while (cmd > 0)

  if (mode == LSBFIRST)                                         ' LSBFIRST result?
    bits ><= count                                              '  reverse bits
   
  return bits


dat { cog code }

                        org     0

entry                   rdlong  r1, par                         ' get packed pins

                        mov     r2, r1                          ' copy pins
                        and     r2, #$3F                        ' isolate dpin
                        mov     dmask, #1                       ' convert to mask
                        shl     dmask, r2

                        mov     r2, r1                          ' copy pins
                        shr     r2, #8                          ' isolate cpin 
                        and     r2, #$3F                        
                        mov     cmask, #1                       ' convert to mask
                        shl     cmask, r2
                        andn    outa, cmask                     ' set to output/low
                        or      dira, cmask

                        mov     r2, r1                          ' copy pins
                        shr     r2, #16                         ' isolate lpin 
                        and     r2, #$3F                        
                        mov     lmask, #1                       ' convert to mask
                        shl     lmask, r2
                        or      outa, lmask                     ' set high (shift)
                        or      dira, lmask                     ' set to ouput mode

clear_cmd               mov     r1, #0                          ' clear command
                        wrlong  r1, par

get_cmd                 rdlong  r1, par                 wz      ' look for command
        if_z            jmp     #get_cmd

                                   
rd165                   andn    outa, lmask                     ' load
                        nop
                        nop
                        or      outa, lmask                     ' shift mode
                        nop
                        mov     x165, #0                        ' clear result var
                        
:loop                   test    dmask, ina              wc      ' read data pin 
                        rcl     x165, #1                        ' move to result, msbfirst
                        or      outa, cmask                     ' clock high
                        nop
                        nop
                        andn    outa, cmask                     ' clock low
                        nop
                        djnz    r1, #:loop

                        mov     hub, par                        ' point to cmd
                        add     hub, #4                         ' point to bits
                        wrlong  x165, hub                       ' save new scan

                        jmp     #clear_cmd              

' ---------------------------------------------------------------------------------------

dmask                   res     1                               ' pin masks
cmask                   res     1
lmask                   res     1

x165                    res     1                               ' result from x165 read

r1                      res     1                               ' working registers
r2                      res     1
r3                      res     1

hub                     res     1                               ' hub pointer

                        fit     496


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}